@@ -4,5 +4,5 @@ from django.utils.translation import gettext_lazy as _
 
 class SalesmanAdminApp(AppConfig):
     name = 'salesman.admin'
-    label = 'salesman'
-    verbose_name = _("Salesman")
+    label = 'salesmanadmin'
+    verbose_name = _("Salesman Admin")
