@@ -1,20 +1,25 @@
 from __future__ import annotations
 
-from typing import Any, Callable
+from typing import TYPE_CHECKING, Any, Callable
 
-from django.db.models import Model
+from django.utils.functional import cached_property
 
-from salesman.orders.status import BaseOrderStatus
+if TYPE_CHECKING:  # pragma: no cover
+    from django.db.models import Model
+
+    from salesman.orders.status import BaseOrderStatus
 
 
-class DefaultSettings:
-    @property
+class AppSettings:
+    @cached_property
     def SALESMAN_PRODUCT_TYPES(self) -> dict:
         """
         A dictionary of product types and their respected serializers
         that are availible for purchase as product. Should be
         formated as ``'app_label.Model': 'path.to.Serializer'``.
         """
+        from salesman.core.typing import Product
+
         product_types = self._setting('SALESMAN_PRODUCT_TYPES', {})
         ret = {}
 
@@ -22,15 +27,16 @@ class DefaultSettings:
             model = self._model(key)
             ret[key] = self._callable(value)
 
-            for attr in ['name', 'code']:
-                if not hasattr(model, attr):
-                    self._error(f"Product type `{key}` must define `{attr}` attribute.")
-
-            if not hasattr(model, 'get_price'):
-                self._error(f"Product type `{key}` must implement `get_price` method.")
+            if not isinstance(model, Product):
+                self._error(
+                    f"Product type `{key}` must subclass `django.db.models.Model` and "
+                    f"implement the `salesman.core.typing.Product` protocol. "
+                    f"Required fields: `id`, `name`, `code`. "
+                    f"Required methods: `get_price(self, request)`. "
+                )
         return ret
 
-    @property
+    @cached_property
     def SALESMAN_BASKET_MODIFIERS(self) -> list:
         """
         A list of strings formated as ``path.to.CustomModifier``.
@@ -59,7 +65,7 @@ class DefaultSettings:
             ret.append(modifier)
         return ret
 
-    @property
+    @cached_property
     def SALESMAN_BASKET_ITEM_VALIDATOR(self) -> Callable:
         """
         A dotted path to basket item validator function.
@@ -86,7 +92,7 @@ class DefaultSettings:
         self._model_label(value)
         return value
 
-    @property
+    @cached_property
     def SALESMAN_PAYMENT_METHODS(self) -> list:
         """
         A list of strings formated as ``path.to.CustomPayment``.
@@ -118,7 +124,7 @@ class DefaultSettings:
             ret.append(payment)
         return ret
 
-    @property
+    @cached_property
     def SALESMAN_ORDER_STATUS(self) -> BaseOrderStatus:
         """
         A dotted path to enum class that contains available order statuses.
@@ -128,6 +134,8 @@ class DefaultSettings:
         returns a dict of statuses with their transitions and ``validate_transition``
         method to validate status transitions.
         """
+        from salesman.orders.status import BaseOrderStatus
+
         default = 'salesman.orders.status.OrderStatus'
         value = self._setting('SALESMAN_ORDER_STATUS', default)
         status = self._callable(value)
@@ -144,7 +152,7 @@ class DefaultSettings:
                 )
         return status
 
-    @property
+    @cached_property
     def SALESMAN_ORDER_REFERENCE_GENERATOR(self) -> Callable:
         """
         A dotted path to reference generator function for new orders.
@@ -154,7 +162,7 @@ class DefaultSettings:
         value = self._setting('SALESMAN_ORDER_REFERENCE_GENERATOR', default)
         return self._callable(value)
 
-    @property
+    @cached_property
     def SALESMAN_ORDER_SERIALIZER(self) -> type:
         """
         A dotted path to a serializer class for Order.
@@ -163,7 +171,7 @@ class DefaultSettings:
         value = self._setting("SALESMAN_ORDER_SERIALIZER", default)
         return self._callable(value)
 
-    @property
+    @cached_property
     def SALESMAN_ORDER_SUMMARY_SERIALIZER(self) -> type:
         """
         A dotted path to a summary serializer class for Order.
@@ -211,7 +219,7 @@ class DefaultSettings:
         self._model_label(value)
         return value
 
-    @property
+    @cached_property
     def SALESMAN_PRICE_FORMATTER(self) -> Callable:
         """
         A dotted path to price formatter function. Function should accept a value
@@ -223,7 +231,7 @@ class DefaultSettings:
         value = self._setting('SALESMAN_PRICE_FORMATTER', default)
         return self._callable(value)
 
-    @property
+    @cached_property
     def SALESMAN_ADDRESS_VALIDATOR(self) -> Callable:
         """
         A dotted path to address validator function. Function should accept a string
@@ -235,7 +243,7 @@ class DefaultSettings:
         value = self._setting('SALESMAN_ADDRESS_VALIDATOR', default)
         return self._callable(value)
 
-    @property
+    @cached_property
     def SALESMAN_EXTRA_VALIDATOR(self) -> Callable:
         """
         A dotted path to extra validator function. Function should accept a dict
@@ -255,7 +263,7 @@ class DefaultSettings:
         """
         return self._setting('SALESMAN_ADMIN_REGISTER', True)
 
-    @property
+    @cached_property
     def SALESMAN_ADMIN_JSON_FORMATTER(self) -> Callable:
         """
         A dotted path to JSON formatter function. Function should accept a dict
@@ -304,4 +312,4 @@ class DefaultSettings:
         return value
 
 
-app_settings = DefaultSettings()
+app_settings = AppSettings()
