@@ -5,4 +5,4 @@ from django.utils.translation import gettext_lazy as _
 class SalesmanCoreApp(AppConfig):
     name = 'salesman.core'
     label = 'salesmancore'
-    verbose_name = _("Core")
+    verbose_name = _("Salesman Core")
